@@ -1,11 +1,12 @@
 package Module::Install::ForC::Env;
 use strict;
 use warnings;
-use Storable ();
-use Config;
-use File::Temp ();
-use POSIX;
-use Text::ParseWords ();
+use Storable ();         # first released with perl 5.007003
+use Config;              # first released with perl 5.00307
+use File::Temp ();       # first released with perl 5.006001
+use POSIX;               # first released with perl 5
+use Text::ParseWords (); # first released with perl 5
+use IPC::Open3;          # first released with perl 5
 
 sub DEBUG () { $ENV{DEBUG} }
 
@@ -15,9 +16,6 @@ sub new {
     # platform specific vars
     my %platformvars = do {
         my %unix = (
-            CC  => $ENV{CC}  || 'gcc',
-            CPP => $ENV{CPP} || 'cpp',
-            CXX => $ENV{CXX} || 'g++',
             PREFIX        => $ENV{PREFIX} || '/usr/',
             LIBPREFIX     => 'lib',
             LIBSUFFIX     => '.a',
@@ -26,10 +24,7 @@ sub new {
             CCDLFLAGS     => ['-fPIC'], # TODO: rename
         );
         my %win32 = (
-            CC  => $ENV{CC}  || 'gcc',
-            CPP => $ENV{CPP} || 'cpp',
-            CXX => $ENV{CXX} || 'g++',
-            PREFIX      => 'C:\\',
+            PREFIX      => $ENV{PREFIX} || 'C:\\',
             LIBPREFIX   => '',
             LIBSUFFIX   => '.lib',
             SHLIBPREFIX => '',
@@ -48,6 +43,9 @@ sub new {
     };
 
     my $opt = {
+        CC  => $ENV{CC}  || $Config{cc}  || 'gcc',
+        CPP => $ENV{CPP} || $Config{cpp} || 'cpp',
+        CXX => $ENV{CXX} || (_has_gplusplus() ? 'g++' : $Config{cc}),
         LD            => $Config{ld},
         LDFLAGS       => '',
         CCFLAGS       => [],
@@ -56,7 +54,7 @@ sub new {
         LIBPATH       => [],
         SHLIBSUFFIX   => '.' . $Config{so},
         RANLIB        => 'ranlib',
-        PROGSUFFIX    => ( $Config{exe_ext} ? ( '.' . $Config{exe_ext} ) : '' ),
+        PROGSUFFIX    => $Config{exe_ext} || '', # $Config{exe_ext} is 'exe' or ''
         CXXFILESUFFIX => [ '.c++', '.cc', '.cpp', '.cxx' ],
         CFILESUFFIX   => ['.c'],
         AR            => $Config{ar},
@@ -80,6 +78,16 @@ sub new {
     return $self;
 }
 
+# do you have g++?
+# @return true if user can execute g++.
+sub _has_gplusplus {
+    my($wtr, $rdr, $err);
+    my $pid = open3($wtr, $rdr, $err, 'g++', '--version') or return 0;
+    my $verstr = <$rdr>;
+    waitpid($pid, 0);
+    return $verstr =~ /^g\+\+/ ? 1 : 0;
+}
+
 # pkg-config
 sub parse_config {
     my ($self, $str) = @_;
@@ -125,8 +133,9 @@ sub try_cc {
         SUFFIX => '.c',
         UNLINK => 1,
     );
+    my $executable = File::Temp->new(UNLINK => 1);
     print $ch $src;
-    my $cmd = "$self->{CC} @{[ $self->_libs ]} @{[ $self->_cpppath ]} @{ $self->{CCFLAGS} } $cfile";
+    my $cmd = "$self->{CC} -o $executable @{[ $self->_libs ]} @{[ $self->_cpppath ]} @{ $self->{CCFLAGS} } $cfile";
     print "$cmd\n" if DEBUG;
     my $exit_status = _quiet_system($cmd);
     WIFEXITED($exit_status) && WEXITSTATUS($exit_status) == 0 ? 1 : 0;
@@ -241,9 +250,10 @@ sub program {
 
     my $ld = $clone->_ld(@$srcs);
 
+    my $target_with_opt = ($self->{CC} =~ /^cl/ && $^O eq 'MSWin32') ? "/out:$target" : "-o $target";
     $self->_push_postamble(<<"...");
 $target: @objects
-	$ld $clone->{LDFLAGS} -o $target @objects @{[ $clone->_libpath ]} @{[ $clone->_libs ]}
+    $ld $clone->{LDFLAGS} $target_with_opt @objects @{[ $clone->_libpath ]} @{[ $clone->_libs ]}
 
 ...
 
@@ -259,7 +269,8 @@ sub _is_cpp {
 }
 
 sub _push_postamble {
-    $Module::Install::ForC::POSTAMBLE .= $_[1];
+    (my $src = $_[1]) =~ s/^[ ]{4}/\t/gmsx;
+    $Module::Install::ForC::POSTAMBLE .= $src;
 }
 
 sub _cpppath {
@@ -274,9 +285,10 @@ sub _compile_objects {
     for my $i (0..@$srcs-1) {
         next if $Module::Install::ForC::OBJECTS{$objects->[$i]}++ != 0;
         my $compiler = $self->_is_cpp($srcs->[$i]) ? $self->{CXX} : $self->{CC};
+        my $object_with_opt = ($compiler =~ /^cl/ && $^O eq 'MSWin32') ? "-c -Fo$objects->[$i]" : "-c -o $objects->[$i]";
         $self->_push_postamble(<<"...");
 $objects->[$i]: $srcs->[$i] Makefile
-	$compiler $opt @{ $self->{CCFLAGS} } @{[ $self->_cpppath ]} -c -o $objects->[$i] $srcs->[$i]
+	$compiler $opt @{ $self->{CCFLAGS} } @{[ $self->_cpppath ]} $object_with_opt $srcs->[$i]
 
 ...
     }
